[![FactSet](https://raw.githubusercontent.com/factset/enterprise-sdk/main/docs/images/factset-logo.svg)](https://www.factset.com)

# OFDB client library for Python

[![API Version](https://img.shields.io/badge/api-v2.1.0-blue)](https://developer.factset.com/api-catalog/ofdb-api)
[![PyPi](https://img.shields.io/pypi/v/fds.sdk.OFDB/0.32.0)](https://pypi.org/project/fds.sdk.OFDB/v/0.32.0)
[![Apache-2 license](https://img.shields.io/badge/license-Apache2-brightgreen.svg)](https://www.apache.org/licenses/LICENSE-2.0)

OFDB API

This Python package is automatically generated by the [OpenAPI Generator](https://openapi-generator.tech) project:

- API version: 2.1.0
- SDK version: 0.32.0
- Build package: org.openapitools.codegen.languages.PythonClientCodegen

For more information, please visit [https://developer.factset.com/contact](https://developer.factset.com/contact)

## Requirements

* Python >= 3.7

## Installation

### Poetry

```shell
poetry add fds.sdk.utils fds.sdk.OFDB==0.32.0
```

### pip

```shell
pip install fds.sdk.utils fds.sdk.OFDB==0.32.0
```

## Usage

1. [Generate authentication credentials](../../../../README.md#authentication).
2. Setup Python environment.
   1. Install and activate python 3.7+. If you're using [pyenv](https://github.com/pyenv/pyenv):

      ```sh
      pyenv install 3.9.7
      pyenv shell 3.9.7
      ```

   2. (optional) [Install poetry](https://python-poetry.org/docs/#installation).
3. [Install dependencies](#installation).
4. Run the following:

> [!IMPORTANT]
> The parameter variables defined below are just examples and may potentially contain non valid values. Please replace them with valid values.

### Example Code

```python
from fds.sdk.utils.authentication import ConfidentialClient

import fds.sdk.OFDB
from fds.sdk.OFDB.api import add_api
from fds.sdk.OFDB.models import *
from dateutil.parser import parse as dateutil_parser
from pprint import pprint

# See configuration.py for a list of all supported configuration parameters.

# Examples for each supported authentication method are below,
# choose one that satisfies your use case.

# (Preferred) OAuth 2.0: FactSetOAuth2
# See https://github.com/FactSet/enterprise-sdk#oauth-20
# for information on how to create the app-config.json file
#
# The confidential client instance should be reused in production environments.
# See https://github.com/FactSet/enterprise-sdk-utils-python#authentication
# for more information on using the ConfidentialClient class
configuration = fds.sdk.OFDB.Configuration(
    fds_oauth_client=ConfidentialClient('/path/to/app-config.json')
)

# Basic authentication: FactSetApiKey
# See https://github.com/FactSet/enterprise-sdk#api-key
# for information how to create an API key
# configuration = fds.sdk.OFDB.Configuration(
#     username='USERNAME-SERIAL',
#     password='API-KEY'
# )

# Enter a context with an instance of the API client
with fds.sdk.OFDB.ApiClient(configuration) as api_client:
    # Create an instance of the API class
    api_instance = add_api.AddApi(api_client)
    path = "path_example" # str | Encode database path
    add_dates = AddDates(
        data=[
            DateAndContent(
                date=20220202,
                content=[
                    SymbolAndFields(
                        key=DataValue(None),
                    ),
                ],
            ),
        ],
    ) # AddDates | Required data for creating date in the database. At least one iterative field is required otherwise it will throw 400.

    try:
        # example passing only required values which don't have defaults set
        api_response = api_instance.add_dates(path, add_dates)

        pprint(api_response)
    except fds.sdk.OFDB.ApiException as e:
        print("Exception when calling AddApi->add_dates: %s\n" % e)

    # # Get response, http status code and response headers
    # try:
    #     api_response, http_status_code, response_headers = api_instance.add_dates_with_http_info(path, add_dates)


    #     pprint(api_response)
    #     pprint(http_status_code)
    #     pprint(response_headers)
    # except fds.sdk.OFDB.ApiException as e:
    #     print("Exception when calling AddApi->add_dates: %s\n" % e)

    # # Get response asynchronous
    # try:
    #     async_result = api_instance.add_dates_async(path, add_dates)
    #     api_response = async_result.get()


    #     pprint(api_response)
    # except fds.sdk.OFDB.ApiException as e:
    #     print("Exception when calling AddApi->add_dates: %s\n" % e)

    # # Get response, http status code and response headers asynchronous
    # try:
    #     async_result = api_instance.add_dates_with_http_info_async(path, add_dates)
    #     api_response, http_status_code, response_headers = async_result.get()


    #     pprint(api_response)
    #     pprint(http_status_code)
    #     pprint(response_headers)
    # except fds.sdk.OFDB.ApiException as e:
    #     print("Exception when calling AddApi->add_dates: %s\n" % e)

```

### Using Pandas

To convert an API response to a Pandas DataFrame, it is necessary to transform it first to a dictionary.
```python
import pandas as pd

response_dict = api_response.to_dict()['data']

simple_json_response = pd.DataFrame(response_dict)
nested_json_response = pd.json_normalize(response_dict)
```

### Debugging

The SDK uses the standard library [`logging`](https://docs.python.org/3/library/logging.html#module-logging) module.

Setting `debug` to `True` on an instance of the `Configuration` class sets the log-level of related packages to `DEBUG`
and enables additional logging in Pythons [HTTP Client](https://docs.python.org/3/library/http.client.html).

**Note**: This prints out sensitive information (e.g. the full request and response). Use with care.

```python
import logging
import fds.sdk.OFDB

logging.basicConfig(level=logging.DEBUG)

configuration = fds.sdk.OFDB.Configuration(...)
configuration.debug = True
```

### Configure a Proxy

You can pass proxy settings to the Configuration class:

* `proxy`: The URL of the proxy to use.
* `proxy_headers`: a dictionary to pass additional headers to the proxy (e.g. `Proxy-Authorization`).

```python
import fds.sdk.OFDB

configuration = fds.sdk.OFDB.Configuration(
    # ...
    proxy="http://secret:password@localhost:5050",
    proxy_headers={
        "Custom-Proxy-Header": "Custom-Proxy-Header-Value"
    }
)
```

### Custom SSL Certificate

TLS/SSL certificate verification can be configured with the following Configuration parameters:

* `ssl_ca_cert`: a path to the certificate to use for verification in `PEM` format.
* `verify_ssl`: setting this to `False` disables the verification of certificates.
  Disabling the verification is not recommended, but it might be useful during
  local development or testing.

```python
import fds.sdk.OFDB

configuration = fds.sdk.OFDB.Configuration(
    # ...
    ssl_ca_cert='/path/to/ca.pem'
)
```

### Request Retries

In case the request retry behaviour should be customized, it is possible to pass a `urllib3.Retry` object to the `retry` property of the Configuration.

```python
from urllib3 import Retry
import fds.sdk.OFDB

configuration = fds.sdk.OFDB.Configuration(
    # ...
)

configuration.retries = Retry(total=3, status_forcelist=[500, 502, 503, 504])
```


## Documentation for API Endpoints

All URIs are relative to *https://api.factset.com/analytics/ofdb/v2*

Class | Method | HTTP request | Description
------------ | ------------- | ------------- | -------------
*AddApi* | [**add_dates**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/OFDB/v2/docs/AddApi.md#add_dates) | **POST** /database/{path}/dates | 
*AddApi* | [**add_fields**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/OFDB/v2/docs/AddApi.md#add_fields) | **POST** /database/{path}/fields | 
*AddApi* | [**add_symbols**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/OFDB/v2/docs/AddApi.md#add_symbols) | **POST** /database/{path}/symbols | 
*CreateApi* | [**create_database**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/OFDB/v2/docs/CreateApi.md#create_database) | **POST** /database | 
*DeleteApi* | [**delete_bulk_items**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/OFDB/v2/docs/DeleteApi.md#delete_bulk_items) | **POST** /database/{path}/delete | 
*DeleteApi* | [**delete_date**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/OFDB/v2/docs/DeleteApi.md#delete_date) | **DELETE** /database/{path}/dates/{date} | 
*DeleteApi* | [**delete_date_from_symbol**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/OFDB/v2/docs/DeleteApi.md#delete_date_from_symbol) | **DELETE** /database/{path}/dates/{date}/symbols/{symbol} | 
*DeleteApi* | [**delete_symbol**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/OFDB/v2/docs/DeleteApi.md#delete_symbol) | **DELETE** /database/{path}/symbols/{symbol} | 
*DeleteApi* | [**get_resource_date_from_symbol**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/OFDB/v2/docs/DeleteApi.md#get_resource_date_from_symbol) | **DELETE** /database/{path}/dates/{date}/symbols/{symbol}/{id} | 
*GetApi* | [**get_audit**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/OFDB/v2/docs/GetApi.md#get_audit) | **GET** /database/{path}/audit | 
*GetApi* | [**get_data**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/OFDB/v2/docs/GetApi.md#get_data) | **GET** /database/{path} | 
*GetApi* | [**get_dates**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/OFDB/v2/docs/GetApi.md#get_dates) | **GET** /database/{path}/dates | 
*GetApi* | [**get_fields**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/OFDB/v2/docs/GetApi.md#get_fields) | **GET** /database/{path}/fields | 
*GetApi* | [**get_stats**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/OFDB/v2/docs/GetApi.md#get_stats) | **GET** /database/{path}/stats | 
*GetApi* | [**get_symbols**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/OFDB/v2/docs/GetApi.md#get_symbols) | **GET** /database/{path}/symbols | 
*LongRunningApi* | [**get_resource_audit**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/OFDB/v2/docs/LongRunningApi.md#get_resource_audit) | **GET** /database/{path}/audit/{id} | 
*LongRunningApi* | [**get_resource_bulk_delete**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/OFDB/v2/docs/LongRunningApi.md#get_resource_bulk_delete) | **GET** /database/{path}/delete/{id} | 
*LongRunningApi* | [**get_resource_data**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/OFDB/v2/docs/LongRunningApi.md#get_resource_data) | **GET** /database/{path}/{id} | 
*LongRunningApi* | [**get_resource_database**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/OFDB/v2/docs/LongRunningApi.md#get_resource_database) | **GET** /database/{id} | 
*LongRunningApi* | [**get_resource_date**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/OFDB/v2/docs/LongRunningApi.md#get_resource_date) | **GET** /database/{path}/dates/{date}/{id} | 
*LongRunningApi* | [**get_resource_dates**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/OFDB/v2/docs/LongRunningApi.md#get_resource_dates) | **GET** /database/{path}/dates/{id} | 
*LongRunningApi* | [**get_resource_fields**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/OFDB/v2/docs/LongRunningApi.md#get_resource_fields) | **GET** /database/{path}/fields/{id} | 
*LongRunningApi* | [**get_resource_stats**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/OFDB/v2/docs/LongRunningApi.md#get_resource_stats) | **GET** /database/{path}/stats/{id} | 
*LongRunningApi* | [**get_resource_symbol**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/OFDB/v2/docs/LongRunningApi.md#get_resource_symbol) | **GET** /database/{path}/symbols/{symbol}/{id} | 
*LongRunningApi* | [**get_resource_symbols**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/OFDB/v2/docs/LongRunningApi.md#get_resource_symbols) | **GET** /database/{path}/symbols/{id} | 
*LongRunningApi* | [**get_status**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/OFDB/v2/docs/LongRunningApi.md#get_status) | **GET** /database/jobs/{id}/status | 
*UpdateApi* | [**update_date**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/OFDB/v2/docs/UpdateApi.md#update_date) | **PUT** /database/{path}/dates/{date} | 
*UpdateApi* | [**update_dates**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/OFDB/v2/docs/UpdateApi.md#update_dates) | **PUT** /database/{path}/dates | 
*UpdateApi* | [**update_symbol**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/OFDB/v2/docs/UpdateApi.md#update_symbol) | **PUT** /database/{path}/symbols/{symbol} | 
*UpdateApi* | [**update_symbols**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/OFDB/v2/docs/UpdateApi.md#update_symbols) | **PUT** /database/{path}/symbols | 


## Documentation For Models

 - [AddDates](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/OFDB/v2/docs/AddDates.md)
 - [AddFields](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/OFDB/v2/docs/AddFields.md)
 - [AddSymbols](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/OFDB/v2/docs/AddSymbols.md)
 - [BulkDelete](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/OFDB/v2/docs/BulkDelete.md)
 - [BulkDeleteArray](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/OFDB/v2/docs/BulkDeleteArray.md)
 - [BulkDeleteArrayItem](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/OFDB/v2/docs/BulkDeleteArrayItem.md)
 - [BulkDeleteArrayItemArray](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/OFDB/v2/docs/BulkDeleteArrayItemArray.md)
 - [BulkDeleteArrayItemNested](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/OFDB/v2/docs/BulkDeleteArrayItemNested.md)
 - [BulkDeleteArrayItemNestedArray](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/OFDB/v2/docs/BulkDeleteArrayItemNestedArray.md)
 - [BulkDeleteArrayItemNestedObject](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/OFDB/v2/docs/BulkDeleteArrayItemNestedObject.md)
 - [BulkDeleteArrayItemObject](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/OFDB/v2/docs/BulkDeleteArrayItemObject.md)
 - [BulkDeleteItem](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/OFDB/v2/docs/BulkDeleteItem.md)
 - [ContentWithDate](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/OFDB/v2/docs/ContentWithDate.md)
 - [ContentWithSymbol](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/OFDB/v2/docs/ContentWithSymbol.md)
 - [CreateDatabase](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/OFDB/v2/docs/CreateDatabase.md)
 - [CreateDatabaseData](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/OFDB/v2/docs/CreateDatabaseData.md)
 - [DataValue](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/OFDB/v2/docs/DataValue.md)
 - [DatabaseSchema](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/OFDB/v2/docs/DatabaseSchema.md)
 - [DatabaseSchemaData](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/OFDB/v2/docs/DatabaseSchemaData.md)
 - [DateAndContent](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/OFDB/v2/docs/DateAndContent.md)
 - [DateAndFields](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/OFDB/v2/docs/DateAndFields.md)
 - [Dates](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/OFDB/v2/docs/Dates.md)
 - [DatesWithId](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/OFDB/v2/docs/DatesWithId.md)
 - [ErrorObject](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/OFDB/v2/docs/ErrorObject.md)
 - [ErrorObjectWithSource](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/OFDB/v2/docs/ErrorObjectWithSource.md)
 - [ErrorObjectWithSourceSource](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/OFDB/v2/docs/ErrorObjectWithSourceSource.md)
 - [FieldProperties](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/OFDB/v2/docs/FieldProperties.md)
 - [Fields](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/OFDB/v2/docs/Fields.md)
 - [InlineResponse200](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/OFDB/v2/docs/InlineResponse200.md)
 - [InlineResponse2001](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/OFDB/v2/docs/InlineResponse2001.md)
 - [InlineResponse2001Data](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/OFDB/v2/docs/InlineResponse2001Data.md)
 - [InlineResponse2002](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/OFDB/v2/docs/InlineResponse2002.md)
 - [InlineResponse2003](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/OFDB/v2/docs/InlineResponse2003.md)
 - [InlineResponse2004](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/OFDB/v2/docs/InlineResponse2004.md)
 - [InlineResponse2005](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/OFDB/v2/docs/InlineResponse2005.md)
 - [InlineResponse2006](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/OFDB/v2/docs/InlineResponse2006.md)
 - [InlineResponse200Data](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/OFDB/v2/docs/InlineResponse200Data.md)
 - [InlineResponse201](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/OFDB/v2/docs/InlineResponse201.md)
 - [InlineResponse201Data](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/OFDB/v2/docs/InlineResponse201Data.md)
 - [InlineResponse202](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/OFDB/v2/docs/InlineResponse202.md)
 - [InlineResponse202Data](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/OFDB/v2/docs/InlineResponse202Data.md)
 - [InlineResponse400](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/OFDB/v2/docs/InlineResponse400.md)
 - [InlineResponse403](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/OFDB/v2/docs/InlineResponse403.md)
 - [SuccessPostResponse](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/OFDB/v2/docs/SuccessPostResponse.md)
 - [SuccessPostResponseData](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/OFDB/v2/docs/SuccessPostResponseData.md)
 - [SymbolAndContent](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/OFDB/v2/docs/SymbolAndContent.md)
 - [SymbolAndFields](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/OFDB/v2/docs/SymbolAndFields.md)
 - [Symbols](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/OFDB/v2/docs/Symbols.md)
 - [SymbolsWithId](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/OFDB/v2/docs/SymbolsWithId.md)
 - [UpdateDate](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/OFDB/v2/docs/UpdateDate.md)
 - [UpdateDates](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/OFDB/v2/docs/UpdateDates.md)
 - [UpdateSymbol](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/OFDB/v2/docs/UpdateSymbol.md)
 - [UpdateSymbols](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/OFDB/v2/docs/UpdateSymbols.md)


## Documentation For Authorization


## FactSetApiKey

- **Type**: HTTP basic authentication


## FactSetOAuth2

- **Type**: OAuth
- **Flow**: application
- **Authorization URL**: 
- **Scopes**: N/A


## Notes for Large OpenAPI documents
If the OpenAPI document is large, imports in fds.sdk.OFDB.apis and fds.sdk.OFDB.models may fail with a
RecursionError indicating the maximum recursion limit has been exceeded. In that case, there are a couple of solutions:

Solution 1:
Use specific imports for apis and models like:
- `from fds.sdk.OFDB.api.default_api import DefaultApi`
- `from fds.sdk.OFDB.model.pet import Pet`

Solution 2:
Before importing the package, adjust the maximum recursion limit as shown below:
```
import sys
sys.setrecursionlimit(1500)
import fds.sdk.OFDB
from fds.sdk.OFDB.apis import *
from fds.sdk.OFDB.models import *
```

## Contributing

Please refer to the [contributing guide](../../../../CONTRIBUTING.md).

## Copyright

Copyright 2022 FactSet Research Systems Inc

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

